import sys
import re
from TOSSIM import *
from random import *
from time import *
from tinyos.tossim.TossimApp import *

def openFile(filename, mode):
  try:
    fd = open(filename, mode);
  except ValuesError:
    print "ValuesError: [readFile] Program exiting...\n";
    os.exit(1);
  except IOError:
    print "IOError: [readFile] Program exiting...\n";
    os.exit(1);
  
  return fd;

def getRcvAddr():
	app = openFile("DSDVMultiHopAppP.nc", "r");

	lines = app.readlines();

	rcv = re.compile("#define BASE_STATION_NODEID(.*)");

	for line in lines:
		foundRcv = rcv.match(line);
		if(foundRcv): 
			break;
	
	return int(foundRcv.group(1).lstrip());

n = NescApp();

t = Tossim(n.variables.variables())
m = t.mac();
r = t.radio();

seed = randint(0, 99999999);
used_seed = ("used seed: %d\n") %(seed);
log_file = ("log%d.txt") % (seed);
res_file = ("res%d.txt") % seed;

t.randomSeed(seed); #20864577 87730701

# Debug Messages
log = open(log_file, "w")

log.write(used_seed);

# Save to file.
t.addChannel("DSDVMultiHopAppP", log )
t.addChannel("DTSN", log);
#t.addChannel("DSDV", log )
#t.addChannel("QueuedSendP", log )
#t.addChannel("MHEngine", log )
#t.addChannel("LedsC", log )
#t.addChannel("DEBUG", log )

# Send to stdout
t.addChannel("DSDVMultiHopAppP", sys.stdout )
t.addChannel("DTSN", sys.stdout);
#t.addChannel("DSDV", sys.stdout )
#t.addChannel("LedsC", sys.stdout )
#t.addChannel("QueuedSendP", sys.stdout )
#t.addChannel("MHEngine", sys.stdout )
#t.addChannel("DEBUG", sys.stdout )

# Number of nodes in the topologie
nNodes = 10

print "Reading the topologie of the network."
# Reading the topologie of the network.
f = open("topo/topo.txt", "r")
lines = f.readlines()
for line in lines:
  s = line.split()
  if (len(s) > 0):
    if s[0] == "gain":
      r.add(int(s[1]), int(s[2]), float(s[3]))
    elif s[0] == "noise":
      r.setNoise(int(s[1]), float(s[2]), float(s[3]))

print "Reading noise values."
# Reading noise values.
noise = open("noise/meyer.txt", "r")
lines = noise.readlines()
for line in lines:
    str = line.strip()
    if (str != ""):
        val = int(str)
        for i in range(0, nNodes):
            t.getNode(i).addNoiseTraceReading(val)

print "Inicialization of the nodes."
# Inicialization of the nodes.
for i in range(0, nNodes):
	m = t.getNode(i);
	time = randint(t.ticksPerSecond(), 1 * t.ticksPerSecond())
	m.bootAtTime(time)
	m.createNoiseModel();
	print "Booting ", i, " at time ", time/t.ticksPerSecond()," seg"
	
print "Starting simulation."
# Running the simulation.

# stats
senderAddr = 0;
RcvAddr = getRcvAddr();
sender = t.getNode(senderAddr);
receiver = t.getNode(RcvAddr);
done = sender.getVariable("DSDVMultiHopAppP.done");

while(done.getData() == 0):
	t.runNextEvent();
	
log.close();

sentToApp = receiver.getVariable("DSDVMultiHopAppP.packetsSentToApp").getData();
genPackets = sender.getVariable("DSDVMultiHopAppP.count").getData();

sentINodes = 0;
for i in range(senderAddr+1, RcvAddr):
	iNode = t.getNode(i);
	print i;
	sentINodes += iNode.getVariable("DSDVMultiHopAppP.sentINode").getData();

pkts = ("\nComm from %d to %d. Sent %d packets to app. Used seed: %d\n") % (senderAddr, RcvAddr, sentToApp, seed);
nc = ("n: %.5f :: sent by snd: %d | sent iNodes: %d\n") % ( float(genPackets+sentINodes)/float(sentToApp), genPackets, sentINodes );
tp = ("PER: %.5f :: sent: %d | rcved: %d \n") % ( (float(genPackets - sentToApp) / float(genPackets)), genPackets, sentToApp );

print pkts, tp, nc,

log = open(log_file, "r");
res = open(res_file, "w");

sent = re.compile('(.*)Sending Message:(.*)');
rcv = re.compile('(.*)Received Message:(.*)');

lines = log.readlines();

sentAt = [];
rcvAt = [];
seqNos = [];
for line in lines:
	foundRcv = rcv.match(line);
		
	if(foundRcv):
		seqNos.append(foundRcv.group(2).split("@")[0]);
		rcvAt.append(foundRcv.group(2).split("@")[1]);

i = 0;
for line in lines:
	foundSent = sent.match(line);
	
	if(foundSent and i < len(seqNos) and seqNos[i] == foundSent.group(2).split("@")[0]):
		sentAt.append(foundSent.group(2).split("@")[1]);
		i += 1;

#print seqNos, sentAt, rcvAt;

for i in range(0, len(rcvAt)):
  exp = ("%d %d %d\n") % (int(seqNos[i]), int(sentAt[i]), int(rcvAt[i]));
  res.writelines(exp);

TxTime = ("Total time: %d ms\n") % (int(rcvAt[len(rcvAt)-1]) - int(sentAt[0]));

print TxTime;

log.close();
res.close();

valFile = open("values.txt", "a");

valFile.writelines(pkts);
valFile.writelines(nc);
valFile.writelines(tp);
valFile.writelines(TxTime);

valFile.close();
