/*
This software licensed to you subject to the following terms and conditions
which you accepted by clicking "Accept" in order to enable the downloading
of this software

1.    In consideration of you agreeing to abide by the terms of this
Agreement, the Licensor hereby grants to you a non-exclusive,
non-transferable license to use the Software and derivations or variations
of the Software on the terms of this Agreement.
2.    You may download, install, use, modify, vary, and adapt the Software.
3.    Except as expressly set out in this Agreement or as permitted by any
mandatory law, you undertake:

    a.    not to copy the Software or except where such copying is
incidental to normal use of the Software, or where it is necessary for the
purpose of back-up or operational security;
    b.    to include the copyright notice of the Licensor on all entire and
partial copies you make of the Software or any variations or derivation of
the Software on any medium;
    c.    not to sublicense the Software or variations or derivations of the
Software to any person without INOV?s written consent.

4.    You acknowledge that all intellectual property rights in the Software,
and any modifications, or variations thereto, anywhere in the world belong
to INOV, or its licensors, that rights in the Software are licensed (not
sold) to you, and that you have no rights in, or to, the Software other than
the right to use them in accordance with the terms of this Agreement.
5.    Nothing in this Agreement shall exclude or in any way limit the
Licensor's liability for fraud, or for death or personal injury caused by
its negligence or any other liability to the extent that it may not be
excluded or limited as a matter of law.
6.    Subject to clause 5  the Licensor shall not be liable under, or in
connection with, this Agreement or any collateral contract for:
    a.    loss of income;
    b.    loss of business profits or contracts;
    c.    business interruption;
    d.    loss of the use of money or anticipated savings;
    e.    loss of information;
    f .    loss of opportunity, goodwill or reputation;
    g.    loss of, damage to or corruption of data; or
    h.    any indirect or consequential loss or damage of any kind howsoever
arising and whether caused by tort (including negligence), breach of
contract or otherwise.
7.    Subject to clauses 5 and 6, the Licensor's maximum aggregate liability
under or in connection with this Agreement, or any collateral contract,
whether in contract, tort (including negligence) or otherwise, shall be
limited to a sum of  100.
8.    This Agreement sets out the full extent of the Licensor's obligations
and liabilities in respect of the supply of the Software. The Software is
provided ?as is?, and there are no conditions, warranties, representations
or other terms, express or implied, that are binding on the Licensor except
as specifically stated in this Agreement.  Any condition, warranty,
representation or other term concerning the supply of the Software which
might otherwise be implied into, or incorporated in, this Agreement, or any
collateral contract, whether by statute, common law or otherwise, is hereby
excluded to the fullest extent permitted by law.
9.    The Licensor may terminate this Agreement on giving you not less than
28 days written notice, or may terminate this Agreement immediately by
written notice to you if:
    a.    You commit a material or persistent breach of this Agreement which
you fail to remedy (if remediable) within 14 days after the service of
written notice requiring you to do so; or
    b.    You undergo a change of control.
10.    Upon termination for any reason:
    a.    all rights granted to you under this Agreement shall cease;
    b.    you must cease all activities authorised by this Agreement;
    c.    you must immediately delete or remove the Software or derivations
or variations thereto from all computer equipment in your possession, and
immediately destroy or return to the Licensor (at the Licensor's option) all
copies of the Software, or derivations or variations thereto, then in your
possession, custody or control and, in the case of destruction, certify to
the Licensor that you have done so.
11.    You may not transfer, assign, charge or otherwise dispose of this
Agreement, or any of your rights or obligations arising under it, without
our prior written consent.
12.    We may transfer, assign, charge, sub-contract or otherwise dispose of
this Agreement, or any of our rights or obligations under it, at any time
during the term of the Agreement.
13.    All notices given by you to us must be given to INOV via e-mail at antonio.grilo@inov.pt.
We may give notice to you at either the e-mail or postal address you
provided to us when registering to download the Software. Notice will be
deemed received and properly served immediately when posted on our website,
24 hours after an e-mail is sent, or three days after the date of posting of
any letter. In proving the service of any notice, it will be sufficient to
prove, in the case of a letter, that such letter was properly addressed,
stamped and placed in the post and, in the case of an e-mail, that such
e-mail was sent to the specified e-mail address of the addressee.
14.    This Agreement and any document expressly referred to in it
represents the entire agreement between us in relation to the licensing of
the Software and supersedes any prior agreement, understanding or
arrangement between us, whether oral or in writing.
15.    A person who is not a party to this Agreement shall have no right
under the Contracts (Rights of Third Parties) Act 1999 to enforce any of its
terms.
16.    INOV reserves the right to change this Agreement at any time without
prior notice. INOV will notify you of any such change or by posting an
updated version of this agreement online. By clicking on the ?Accept? button
at the bottom of an updated version you will agree to the terms of the
varied version which will bind you and your employees.
17.    This Agreement is governed by English law. Any dispute arising from,
or related to, any term of this Agreement shall be subject to the
non-exclusive jurisdiction of the courts of Portugal.

author          = "Filipe Alves, Antonio Grilo",
date            = "17-10-2008",
filename        = "dsdv.h",
address         = "INOV, Rua Alves Redol, No. 9, 1000-029 Lisboa, Portugal",
telephone       = "+351 213100226",
FAX             = "+351 213100226",
email           = "antonio.grilo@inov.pt",
description = " This files contains the message types and header structures for the DSDV routing algorithm."
 
Copyright (c) 2008 INOV.
All Rights Reserved
*/


#ifndef _MULTIHOP_DSDV_H_
#define _MULTIHOP_DSDV_H_

#define DSDVNOBROADCASTS

// Configuration Variables.
enum {
	DSDV_MULTIHOPMSG    = 250,    // Routing  hello message active id.
	ROUTE_TABLE_SIZE    = 17,     // Routing table size.
	DELAY		    = 1000,   // Delay to send first hello messages.
	UPDATE_INTERVAL	    = 6000, // Time interval for update of routing table. Default 5 min.
	UPDATE_JITTER				= 1000,
	TIME_OUT            = 2,     // Time out to discart a table entry.
	ROUTE_INVALID       = 0xff,   // Constant to tell that no route was found.
	DSDV_RECEIVE_QUEUE  = 5	      // Size of queue for retransminting incoming hello messages.
};

// Messages Structures definitions.

typedef struct {
	am_addr_t sourceaddr;
	am_addr_t destinyaddr;
//	nx_uint8_t data[TOSH_DATA_LENGTH-4];
} RouteHeader;

typedef struct {
	am_addr_t destinyaddr;
	am_addr_t parentaddr;
	uint16_t hopnumber;
	int8_t seqno;
} RoutePacket;

typedef struct {
	RoutePacket entry;
	uint8_t timeout;
} TableEntry;

// Debug Functions.
void debugRoutePacket( RoutePacket *msg ){
	
	dbg("DSDV", "\tRoutePacket:\n");
	dbg("DSDV", "\t\tparentaddr: %d\n", msg->parentaddr);
	dbg("DSDV", "\t\tdestinyaddr: %d\n", msg->destinyaddr);
	dbg("DSDV", "\t\tseqno: %d\n", msg->seqno);
	dbg("DSDV", "\t\thopcount: %d\n", msg->hopnumber );	
}

void debugRoutingTableDTSN(TableEntry NeighborTbl[] ){

	int i;
	dbg("DTSN","\t|Dest.\t|SeqNo.\t|Next\t|# Hops\t|Timeout|\n");
	for (i = 0;i < ROUTE_TABLE_SIZE;i++) {
		//dbg("DTSN", "[DSDV] timeout: %d\n", NeighborTbl[i].timeout);
		if (NeighborTbl[i].timeout != 0 ) {
			dbg("DTSN","\t|%d\t|%d\t|%d\t|%d\t|%d\t|\n",
				NeighborTbl[i].entry.destinyaddr,
				NeighborTbl[i].entry.seqno,
				NeighborTbl[i].entry.parentaddr,
				NeighborTbl[i].entry.hopnumber,
				NeighborTbl[i].timeout );
		}
	}
}

void debugRoutingTable(TableEntry NeighborTbl[] ){

	int i;
	dbg("DSDV","\t|Dest.\t|SeqNo.\t|Next\t|# Hops\t|Timeout|\n");
	for (i = 0;i < ROUTE_TABLE_SIZE;i++) {
		//dbg("DTSN", "[DSDV] timeout: %d\n", NeighborTbl[i].timeout);
		if (NeighborTbl[i].timeout != 0 ) {
			dbg("DSDV","\t|%d\t|%d\t|%d\t|%d\t|%d\t|\n",
				NeighborTbl[i].entry.destinyaddr,
				NeighborTbl[i].entry.seqno,
				NeighborTbl[i].entry.parentaddr,
				NeighborTbl[i].entry.hopnumber,
				NeighborTbl[i].timeout );
		}
	}
}

void debugRouteHeader( RouteHeader *header ){

	dbg( "DSDVHeader", "\tRouteHeader:\n");
	dbg( "DSDVHeader", "\t\tsourceaddr : %d\n", header->sourceaddr );
	dbg( "DSDVHeader", "\t\tdestinyaddr: %d\n", header->destinyaddr);
	
}

#endif
